"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Predefined relative difference/error scalers.
 */
var IsCloseScalers = /** @class */ (function () {
    function IsCloseScalers() {
    }
    /**
     * Relative difference scaling: Max(|a|,|b|)
     * @param {number} a
     * @param {number} b
     * @returns {number}
     */
    IsCloseScalers.maxAbsAOrB = function (a, b) {
        return Math.max(Math.abs(a), Math.abs(b));
    };
    /**
     * Relative difference scaling: Max(a,b)
     * @param {number} a
     * @param {number} b
     * @returns {number}
     */
    IsCloseScalers.maxAOrB = function (a, b) {
        return Math.max(a, b);
    };
    /**
     * Relative difference scaling: Min(|a|,|b|)
     * @param {number} a
     * @param {number} b
     * @returns {number}
     */
    IsCloseScalers.minAbsAOrB = function (a, b) {
        return Math.min(Math.abs(a), Math.abs(b));
    };
    /**
     * Relative difference scaling: Min(a,b)
     * @param {number} a
     * @param {number} b
     * @returns {number}
     */
    IsCloseScalers.minAOrB = function (a, b) {
        return Math.min(a, b);
    };
    /**
     * Relative difference scaling: Mean: (|a|+|b|)/2, using Welford's method
     * @param {number} a
     * @param {number} b
     * @returns {number}
     */
    IsCloseScalers.meanAbs = function (a, b) {
        a = Math.abs(a);
        b = Math.abs(b);
        return a + (b - a) / 2.0;
    };
    /**
     * Relative difference scaling: Mean: (a+b)/2, using Welford's method
     * @param {number} a
     * @param {number} b
     * @returns {number}
     */
    IsCloseScalers.mean = function (a, b) {
        return a + (b - a) / 2.0;
    };
    /**
     * Relative error scaling: abs(a)
     * @param {number} a Treated as theoretical expected value for calculating relative error
     * @returns {number}
     */
    IsCloseScalers.absA = function (a) {
        return Math.abs(a);
    };
    /**
     * Relative error scaling: abs(b)
     * @param {number} a ignored
     * @param {number} b Treated as theoretical expected value for calculating relative error
     * @returns {number}
     */
    IsCloseScalers.absB = function (a, b) {
        return Math.abs(b);
    };
    return IsCloseScalers;
}());
exports.IsCloseScalers = IsCloseScalers;
