//////////////////////////////////////////////////////
//  Written by Arthur Goins of SuperDroid Robots    //
//                                                  //
//                                                  //
//////////////////////////////////////////////////////

// Set up deps
const express = require('express');
const app = express();
var SerialPort = require("serialport");
var ReadLine = SerialPort.parsers.Readline;
var parser = new ReadLine();


// State of digial inputs, analog inputs, and camera centering button (digitals swap between '0' and '1', analogs go from 0 - 127 with 63 as stopped, 64-127 is positive, and 0-62 is negative)
var sendData = ['0','0','0','0', 63, 63, 63, 63, '0', 'c'];

var timeout = 15;   // 1.5 second timeout
var port = 3000;    // port for REST API
var ardPacket = ""; // Store return packet from Arduino

var arduinoCOMPort = "/dev/ttyUSB0";  // Serial port for Arduino

// Open serial port for Arduino at 9600 baud
var arduinoSerialPort = new SerialPort(arduinoCOMPort, {  
  baudRate: 9600
});
arduinoSerialPort.pipe(parser)
// Start Timeout interval when serial port opens
arduinoSerialPort.on('open',function() {
  console.log('Serial Port ' + arduinoCOMPort + ' is opened.');
  setInterval(() => {
      if(timeout > 0){
          timeout--;
      }
      else{
          sendData = ['0','0','0','0', 63, 63, 63, 63, '0', 'c']; // Failsafe state
          arduinoSerialPort.write(sendData[0]+sendData[1]+sendData[2]+sendData[3] + String.fromCharCode(sendData[4]) +  String.fromCharCode(sendData[5]) + String.fromCharCode(sendData[6]) + String.fromCharCode(sendData[7]) + sendData[8] + sendData[9] + '\n');
      }
  }, 100);
});

// Get list of networks
var exec = require('child_process').exec;
function execute(command, callback){
    exec(command, function(error, stdout, stderr){ callback(stdout); });
}

// Parse incoming data and split it apart by the "|" character
parser.on('data', data =>{
  var parsedData = data.split("|");
  parsedData[0] = parseInt(parsedData[0])*55/255;
  // Set return packet
  if(JSON.stringify(parsedData).length > 0){
    ardPacket = JSON.stringify(parsedData);
  }
});

// Get request handler
app.get('/', function (req, res) {
  // reset timeout
  timeout = 15;
  // Set security header
  res.header('Access-Control-Allow-Origin', '*');
  // Store and parse query data
  var state = req.query.state;
  var drive = sendData[4] + (req.query.drive - sendData[4])/4;
  var turn = sendData[5] + (req.query.turn - sendData[5])/4;
  var pan = sendData[6] + (req.query.pan - sendData[6]);
  var tilt = sendData[7] + (req.query.tilt - sendData[7]);
  sendData = [req.query.led0, req.query.led1, req.query.led2, req.query.led3, drive, turn, pan, tilt, req.query.center, 'c'];
  // Send control data to Arduino
  arduinoSerialPort.write(sendData[0]+sendData[1]+sendData[2]+sendData[3] + String.fromCharCode(sendData[4]) +  String.fromCharCode(sendData[5]) + String.fromCharCode(sendData[6]) + String.fromCharCode(sendData[7]) + sendData[8] + sendData[9] + '\n');
  // Send return packet
  return res.send(ardPacket);
});

// Listen for get requests
app.listen(port, function () {
  console.log('Example app listening on port http://0.0.0.0:' + port + '!');
});

