//////////////////////////////////////////////////////
//  Written by Arthur Goins of SuperDroid Robots    //
//                                                  //
//             Modified for Order 64451             //
//////////////////////////////////////////////////////

// State of digial inputs, analog inputs, and camera centering button 
// // digitals swap between '0' and '1', analogs go from 0 - 127 with 63 as stopped, 64-127 is positive, and 0-62 is negative)
var switchStates = ['0','0','0','0', 63, 63, 63, 63, '0'];  

var inputStates = [false, false, false, false];             // Variable to track state of buttons on display
var responseArr = [];                                       // Response from robot, containing battery voltage and encoder counts
var battVolt = 0;                                           // Battery Voltage (set by response packet)
var touch1 = false;                                         // State of joystick 1
var touch2 = false;                                         // State of joystick 2
var setSlowspeed = 0.5;                                         // Adjust this to change the speed scaling of the drive and turn controls
var setFastspeed = 1;

var setspeed = 0.5;
var slowMode = true;
var sprayMode = false;
var startInt = 0;
var virtButtons = [false, false, false, false];
var sprayInt;
var spraying;

function swapSpeed(){
    if(slowMode){
        document.getElementById("speedSwap").src = "images/fast.png";
        setspeed = setFastspeed;
        slowMode = false;
    }
    else{
        document.getElementById("speedSwap").src = "images/slow.png";
        setspeed = setSlowspeed;
        slowMode = true;
    }
}

// Callback for handling robot response (parsing battery voltage and encoder counts)
function packetCallback(){
    battVolt = battVolt + (responseArr[0] - battVolt)/10;
    // 100%
    if (battVolt >= 25.46){
        document.getElementById("battery").src = "images/batteryfull.png";
    }
    // 75%
    else if(battVolt < 25.46 && battVolt >= 24.87){
        document.getElementById("battery").src = "images/battery75.png";
    }
    // 50%
    else if(battVolt < 24.87 && battVolt >= 24.2){
        document.getElementById("battery").src = "images/battery50.png";
    }
    // 25%
    else if(battVolt < 24.2 && battVolt >= 23.47){
        document.getElementById("battery").src = "images/battery25.png";
    }
    // <25%
    else if (battVolt < 23.47){
        document.getElementById("battery").src = "images/batterylow.png";
    }

}

// Send center command for 0.5 seconds
function centerCam(){
    switchStates[8] = '1';
    sendPacket();
    setTimeout(function(){
        switchStates[8] = '0'
    },500);
}

// Send control state packet and handle response
function sendPacket(){
    // console.log(switchStates)
    if(switchStates[4] < 75 && startInt != 0){
        clearInterval(spraying);
    }
    const Http = new XMLHttpRequest();
    // Split up control packet into query parameters
    var url = "";
    if(sprayMode && switchStates[4]>65){
        switchStates[4] = 96;
    }
    url="http://" + window.location.host + ':3000/?led0=' + switchStates[0] + "&led1=" + switchStates[1] + "&led2=" + switchStates[2] + "&led3=" + switchStates[3] + "&drive=" + switchStates[4] + "&turn=" + switchStates[5] + "&pan=" + switchStates[6] + "&tilt=" + switchStates[7] + "&center=" + switchStates[8]; 
    Http.open("GET", url);
    Http.send();
    Http.onreadystatechange=(e)=>{
        try{
            responseArr = JSON.parse(Http.responseText);
            // console.log(responseArr);
            packetCallback();
        }
        catch{}
    }
}


// Constructor for Joystick 1 (drive and turn)
function createJoystick1() {
        joystickContainer = document.getElementById('joystick1');
        
        // Set up options for dynamic joystick
        var options = {
            zone: joystickContainer,
            mode: 'dynamic',
            size: 150,
            color: '#0066ff'
        };
        options.size = window.innerHeight/6;
        manager1 = nipplejs.create(options);
        
        // Event triggered when joystick is moved
        manager1.on('move', function (evt, nipple) {
            // Set Joystick 1 state
            touch1 = true;

            // Get direction of movement
            var direction = nipple.angle.degree - 90;
            if (direction > 180) {
                direction = -(450 - nipple.angle.degree);
            }

            // Find how far the joystick is being dragged and cap it off at 2x the joystick container size
            var speed = nipple.force;
            if(speed > 2){
                speed = 2;
            }
            if(speed < -2){
                speed = -2;
            }

            // Scale direction and distance to 0-127 for drive and turn
            var drive = setspeed*speed*Math.cos((Math.PI/180)*direction);
            drive = Math.round(drive*62/2 + 63);

            if(Math.abs(drive - 63) < 5){
                drive = 63;
            }

            // Set drive and turn values in control packet
            switchStates[4] = drive;
            
        });

        // Event triggered on release
        manager1.on('end', function () {
            // Reset Joystick 1 state
            touch1 = false;

            // Reset drive and turn values
            switchStates[4] = 63;
        });
}

// Constructor for Joystick 2 (pan and tilt)
function createJoystick2() {
        joystickContainer = document.getElementById('joystick2');

        // Set options for dynamic joystick
        var options = {
            zone: joystickContainer,
            mode: 'dynamic',
            size: 150,
            color: '#0066ff'
        };
        options.size = window.innerHeight/6;
        manager2 = nipplejs.create(options);

        // Event triggered when joystick is moved
        manager2.on('move', function (evt, nipple) {
            // Set state of Joystick 2
            touch2 = true;

            // Get direction of movement
            var direction = nipple.angle.degree - 90;
            if (direction > 180) {
                direction = -(450 - nipple.angle.degree);
            }

            // Find how far the joystick is being dragged and cap it off at 2x the joystick container size
            var speed = nipple.force;
            if(speed > 2){
                speed = 2;
            }
            if(speed < -2){
                speed = -2;
            }

            var turn = speed*Math.sin((Math.PI/180)*direction);
            turn = Math.round(turn*62/2 + 63);

            // Create deadzone 2 wide
            if(Math.abs(turn - 63) < 1){
                turn = 63;
            }

            switchStates[5] = turn;
        });
        // Event triggered on release
        manager2.on('end', function () {
            // Reset Joystick 1 state
            touch2 = false;
            switchStates[5] = 63;
        });
}

// Init function
window.onload = (function(){
    // document.getElementById("videoPlayer").src= ("http://" + window.location.host + ":8888");
    // Construct Joysticks
    createJoystick1();
    createJoystick2();
    // Set up double click to reset encoders
    // document.getElementById("encoders").addEventListener('dblclick', function(e) {
    //     resetEncs();
    // })
    // Start 80ms loop for sending the control packet
    setInterval(function(){
        sendPacket()
    },80)

    // Start 100ms loop for polling the gamepad
    setInterval(() => {
            // If neither joystick is being used, check gamepad for controls
            if(!(touch1||touch2)){
                // Use first item from list of gamepads (will be undefined if no gamepads are connected)
                var myGamepad;
                try{
                    myGamepad = navigator.getGamepads()[0]; 
                }
                catch{
                    
                }
                if(myGamepad === null){
                    myGamepad = {
                        "buttons":[
                            {"pressed": virtButtons[0]},
                            {"pressed": virtButtons[1]},
                            {"pressed": virtButtons[2]},
                            {"pressed": virtButtons[3]}
                        ]
                    }
                }
                // console.log(myGamepad);
                // Green = 0
                // Red = 1
                // Yellow = 3
                // Blue = 2
                if((myGamepad.buttons[0].pressed || virtButtons[0]) && sprayMode == 0){
                    toggleSpray(1); 
                }
                else if((myGamepad.buttons[1].pressed || virtButtons[1]) && sprayMode == 1){
                    toggleSpray(0);
                }
                if(myGamepad.buttons[2].pressed || virtButtons[2]){
                    switchStates[3] = '1';
                }
                if(myGamepad.buttons[3].pressed || virtButtons[3]){
                    switchStates[0] = '1';
                }
                if(!myGamepad.buttons[2].pressed && !virtButtons[2]){
                    switchStates[3] = '0';
                }
                if((!myGamepad.buttons[3].pressed && !virtButtons[3]) && sprayMode == 0){
                    switchStates[0] = '0';
                }
                
                // Try will break here if no gamepad is connected
                // Fetch control values and set deadzone
                try{
                    var dr = -myGamepad.axes[1]*63*setspeed + 63;
                    if (Math.abs(dr - 63) < 2){
                        dr = 63;
                    }
                    var tu = -myGamepad.axes[2]*63*setspeed + 63;
                    if (Math.abs(tu - 63) < 2){
                        tu = 63;
                    }
                    // Round values and set control state packet
                    switchStates[4] = Math.round(dr);
                    switchStates[5] = Math.round(tu);
                }
                catch{

                }
                
            }
        
    }, 100)
})

function toggleSpray(mode){
    if(mode == 0){
        sprayMode = false;
        clearInterval(sprayInt);
    }
    else{
        sprayMode = true;
        sprayInt = setInterval(() => {
            // console.log(startInt)
            if (switchStates[4] >= 65){
                if(startInt == 0 ){
                    startInt = 1;
                }
            }
            if(switchStates[4] < 65){
                startInt = 0;
            }
            if(startInt == 1 && sprayMode){
                switchStates[0] = '1';
                spray()
                startInt = 2;
            }
            else if (startInt == 0 && sprayMode){
                try{
                    clearInterval(spraying);
                    switchStates[0] = '0';
                }
                catch{}
            }
        }, 100);
        
    }
}

async function spray(){
    console.log("SPRAYING_START")
    sprayFunc();
    spraying = setInterval(() => {
        sprayFunc();
    }, 2200);
}

function sprayFunc() {
    console.log("SPRAYING")
    if(sprayMode){
        switchStates[1] = '1';
        setTimeout(() => {
            switchStates[1] = '0';
            setTimeout(() => {
                switchStates[2] = '1';
                setTimeout(() => {
                    switchStates[2] = '0';
                }, 1000);
            }, 100);
        }, 1000);
    }
    else{
        switchStates[1] = '0';
        switchStates[2] = '0';
    }
    
}